/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.16.1.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for C8051F930DK eval board
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
__sfr __no_init volatile unsigned char P1      @ 0x90; // Port1 data register
__sfr __no_init volatile unsigned char P1MDIN  @ 0xF2; // Port1 input mode register
__sfr __no_init volatile unsigned char P1MDOUT @ 0xA5; // Port1 output mode register
__sfr __no_init volatile unsigned char XBR2    @ 0xE3; // Port I/O crossbar register 2

#define LEDO  (1u << 5u)  // P1.5 LED DS3 red
#define LED1  (1u << 6u)  // P1.6 LED DS3 yellow

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  P1MDIN  |= (LEDO | LED1);
  P1MDOUT |= (LEDO | LED1);
  P1      |= (LEDO | LED1);
  XBR2    |= 0x40;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 2) {
    P1 &= ~(1u << (Index + 5));
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 2) {
    P1 |= (1u << (Index + 5));
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < 2) {
    P1 ^= (1u << (Index + 5));
  }
}

/*************************** End of file ****************************/
